// zombie.cpp : Defines the entry point for the DLL application.
// (C) Copyright 2010 Tony Bruess
// Please see the README

#include "bzfsAPI.h"
#include "plugin_utils.h"
#include "StateDatabase.h"
#include "../src/bzfs/GameKeeper.h"
#include "../src/bzfs/bzfs.h"
#include "../src/bzfs/CmdLineOptions.h"
#include <cstdlib>
#include <iostream>

bool gameInProgress = false;
int timer;
double lastsecond;
double lastsec;
std::map<int, bz_eTeamType> teamtracker;

extern CmdLineOptions *clOptions;
extern uint16_t curMaxPlayers;
extern TeamInfo team[NumTeams];

BZ_GET_PLUGIN_VERSION

class zombieHandler : public bz_EventHandler, public bz_CustomSlashCommandHandler
{
public:
    virtual void process(bz_EventData *eventData);
    virtual bool handle(int playerID, bzApiString command, bzApiString message, bzAPIStringList* /*params*/);
};

zombieHandler zombie;

BZF_PLUGIN_CALL int bz_Load(const char* /*commandLine*/)
{
    bz_registerEvent(bz_ePlayerDieEvent, &zombie);
    bz_registerEvent(bz_ePlayerSpawnEvent, &zombie);
    bz_registerEvent(bz_ePlayerJoinEvent, &zombie);
    bz_registerEvent(bz_eGetPlayerSpawnPosEvent, &zombie);
    bz_registerEvent(bz_eFlagDroppedEvent, &zombie);
    bz_registerEvent(bz_eTickEvent, &zombie);
    bz_registerEvent(bz_eGetAutoTeamEvent, &zombie);
    bz_registerEvent(bz_ePlayerPartEvent, &zombie);

    bz_registerCustomSlashCommand ("start", &zombie);
    bz_registerCustomSlashCommand ("stop", &zombie);
    bz_registerCustomSlashCommand ("switch", &zombie);
    bz_registerCustomSlashCommand ("restart", &zombie);
    bz_registerCustomSlashCommand ("join", &zombie);
    bz_registerCustomSlashCommand ("debug", &zombie);

    bz_debugMessage(4,"zombie plugin loaded");

    return 0;
}

bool isAdmin(int playerID)
{
    bz_PlayerRecord* player = bz_getPlayerByIndex(playerID);
    bool admin = player->hasPerm("ban");
    bz_freePlayerRecord(player);
    return admin;
}

int getPlayerByCallsign(bzApiString callsign)
{
    bzAPIIntList* players = bz_newIntList();
    bz_getPlayerIndexList(players);
    int id = -1;
    for(unsigned int i = 0; i < players->size(); i++)
    {
        bz_PlayerRecord* player = bz_getPlayerByIndex(players->get(i));
        bool found = false;
        if(player->callsign == callsign)
        {
            id = (int)i;
            found = true;
        }
        bz_freePlayerRecord(player);
        if(found)
            break;
    }
    bz_deleteIntList(players);
    return id;
}

void fixTeamCount()
{
    int playerIndex, teamNum;
    for (teamNum = RogueTeam; teamNum < RabbitTeam; teamNum++)
        team[teamNum].team.size = 0;
    for (playerIndex = 0; playerIndex < curMaxPlayers; playerIndex++)
    {
        GameKeeper::Player *p = GameKeeper::Player::getPlayerByIndex(playerIndex);
        if (p && p->player.isPlaying())
        {
            teamNum = p->player.getTeam();
            if (teamNum == RabbitTeam)
                teamNum = RogueTeam;
            team[teamNum].team.size++;
        }
    }
}

void addPlayer(GameKeeper::Player *playerData, int index)
{
    void *bufStart = getDirectMessageBuffer();
    void *buf = playerData->packPlayerUpdate(bufStart);
    if (playerData->getIndex() == index)
    {
        broadcastMessage(MsgAddPlayer, (char*)buf - (char*)bufStart, bufStart);
    }
    else
    {
        directMessage(index, MsgAddPlayer, (char*)buf - (char*)bufStart, bufStart);
    }
    int teamNum = int(playerData->player.getTeam());
    team[teamNum].team.size++;
    sendTeamUpdate(-1, teamNum);
    fixTeamCount();
}

void removePlayer(int playerIndex)
{
    GameKeeper::Player *playerData = GameKeeper::Player::getPlayerByIndex(playerIndex);
    if (!playerData)
        return;
    void *buf, *bufStart = getDirectMessageBuffer();
    buf = nboPackUByte(bufStart, playerIndex);
    broadcastMessage(MsgRemovePlayer, (char*)buf-(char*)bufStart, bufStart);
    int teamNum = int(playerData->player.getTeam());
    --team[teamNum].team.size;
    sendTeamUpdate(-1, teamNum);
    fixTeamCount();
}

bool switchPlayer(int playerID, bz_eTeamType team)
{
    GameKeeper::Player *playerData = GameKeeper::Player::getPlayerByIndex(playerID);
    if (!playerData)
        return true;
    removePlayer(playerID);
    playerData->player.setTeam((TeamColor)team);
    addPlayer(playerData, playerID);
    return true;
}

void setVar(int playerID, const string& key, const string& value)
{
    void *bufStart = getDirectMessageBuffer();
    void *buf = nboPackUShort(bufStart, 1);
    buf = nboPackUByte (buf, key.length());
    buf = nboPackString(buf, key.c_str(), key.length());
    buf = nboPackUByte (buf, value.length());
    buf = nboPackString(buf, value.c_str(), value.length());
    const int len = (char*)buf - (char*)bufStart;
    directMessage(playerID, MsgSetVar, len, bufStart);
}

void setZombie(int playerID)
{
    setVar(playerID,"_tankSpeed","35");
    setVar(playerID,"_reloadTime","0.0000001");
    setVar(playerID,"_radarLimit","400");
    setVar(playerID,"_tankAngVel","0.785398");
    bz_givePlayerFlag(playerID, "SR", false);
    switchPlayer(playerID, eGreenTeam);
}

void setHuman(int playerID)
{
    setVar(playerID,"_tankSpeed","25");
    setVar(playerID,"_reloadTime","2.5");
    setVar(playerID,"_radarLimit","100");
    setVar(playerID,"_tankAngVel","0.785398");
    bz_removePlayerFlag(playerID);
    switchPlayer(playerID, eRedTeam);
}

void setRogue(int playerID)
{
    setVar(playerID,"_tankSpeed","0.00000001");
    setVar(playerID,"_reloadTime","0.00000001");
    setVar(playerID,"_radarLimit","0");
    setVar(playerID,"_tankAngVel","0.00000001");
    bz_removePlayerFlag(playerID);
    switchPlayer(playerID, eRogueTeam);
}

void setObserver(int playerID)
{
    setVar(playerID,"_tankSpeed","25");
    setVar(playerID,"_reloadTime","3.5");
    setVar(playerID,"_radarLimit","800");
    setVar(playerID,"_tankAngVel","0.785398");
    bz_removePlayerFlag(playerID);
    switchPlayer(playerID, eObservers);
}

void dontMove(int playerID)
{
    setVar(playerID,"_tankSpeed","0.00000001");
    setVar(playerID,"_reloadTime","0.00000001");
    setVar(playerID,"_radarLimit","0");
    setVar(playerID,"_tankAngVel","0.00000001");
    bz_removePlayerFlag(playerID);
}

void zombieHandler::process(bz_EventData *eventData)
{
    switch (eventData->eventType)
    {
    default:
    break;
    case bz_ePlayerDieEvent:
    {
        bz_PlayerDieEventData* diedata = (bz_PlayerDieEventData*)eventData;
        bz_PlayerRecord *player = bz_getPlayerByIndex(diedata->playerID);
        bz_PlayerRecord *killer = bz_getPlayerByIndex(diedata->killerID);
        if(diedata->team == eRedTeam && diedata->killerTeam == eGreenTeam && gameInProgress)
        {
            bz_sendTextMessage(BZ_SERVER, diedata->playerID, "You are now a zombie -- Go kill those humans!");
            teamtracker[diedata->playerID] = eGreenTeam;
            bz_debugMessagef(1,"ZOMBIEDEBUG: Player: %s || Killer: %s || Player team: red || Killer team: green", player->callsign.c_str(), killer->callsign.c_str());
        }
        if(diedata->team == eGreenTeam && diedata->killerTeam == eRedTeam && gameInProgress)
        {
            bz_sendTextMessage(BZ_SERVER, diedata->playerID, "You are now a human -- Go kill those zombies!");
            teamtracker[diedata->playerID] = eRedTeam;
            bz_debugMessagef(1,"ZOMBIEDEBUG: Player: %s || Killer: %s || Player team: green || Killer team: red", player->callsign.c_str(), killer->callsign.c_str());
        }
        bz_freePlayerRecord(player);
        bz_freePlayerRecord(killer);
    }
    break;
    case bz_ePlayerJoinEvent:
    {
        bz_PlayerJoinPartEventData* joindata = (bz_PlayerJoinPartEventData*)eventData;
        bz_PlayerRecord *player = bz_getPlayerByIndex(joindata->playerID);
        bz_debugMessagef(1,"ZOMBIEDEBUG: %s joining from %f", player->callsign.c_str(), player->ipAddress.c_str());
        if(gameInProgress || joindata->team == eObservers)
        {
            bz_debugMessagef(1,"ZOMBIEDEBUG: %s joined to the observer team", player->callsign.c_str());
            if(gameInProgress)
            {
                bz_sendTextMessage(BZ_SERVER, joindata->playerID, "There is currently a game in progress. Wait untill the game is done, then type /join");
            }
            else
            {
                bz_sendTextMessage(BZ_SERVER, joindata->playerID, "The game has not started. If you would like to join, type /join.");
            }
        }
        else
        {
            bz_debugMessagef(1,"ZOMBIEDEBUG: %s joined to the rogue team", player->callsign.c_str());
            dontMove(joindata->playerID);
            bz_sendTextMessage(BZ_SERVER, joindata->playerID, "To start the game, wait untill there are at least three players, then type /start");
        }
        bz_freePlayerRecord(player);
    }
    break;
    case bz_ePlayerSpawnEvent:
    {
        bz_PlayerSpawnEventData* spawndata = (bz_PlayerSpawnEventData*)eventData;
        bz_PlayerRecord *player = bz_getPlayerByIndex(spawndata->playerID);
        if(spawndata->team == eGreenTeam && gameInProgress)
        {
            bz_givePlayerFlag(spawndata->playerID,"SR",true);
            bz_debugMessagef(1,"ZOMBIEDEBUG: %s was spawned as was given a SR flag", player->callsign.c_str());
        }
        bz_freePlayerRecord(player);
    }
    break;
    case bz_eFlagDroppedEvent:
    {
        bz_FlagDroppedEventData* flagdropdata = (bz_FlagDroppedEventData*)eventData;
        bz_givePlayerFlag(flagdropdata->playerID, flagdropdata->flagType, true);
    }
    break;
    case bz_eGetAutoTeamEvent:
    {
        bz_GetAutoTeamEventData *autoteamdata = (bz_GetAutoTeamEventData*)eventData;
        bz_PlayerRecord *player = bz_getPlayerByIndex(autoteamdata->playeID);
        if(gameInProgress || autoteamdata->team == eObservers || timer == 1)
        {
            teamtracker[autoteamdata->playeID] = eObservers;
            autoteamdata->team = eObservers;
            bz_debugMessagef(1,"ZOMBIEDEBUG: %s was set as an observer via autoteam", player->callsign.c_str());
        }
        else
        {
            teamtracker[autoteamdata->playeID] = eRogueTeam;
            autoteamdata->team = eRogueTeam;
            bz_debugMessagef(1,"ZOMBIEDEBUG: %s was set as a rogue via autoteam", player->callsign.c_str());
        }
        autoteamdata->handled = true;
        bz_freePlayerRecord(player);
    }
    break;
    case bz_eTickEvent:
    {
        bz_TickEventData* tickdata = (bz_TickEventData*)eventData;
        if(timer!=0)
        {
            if(tickdata->time-1>lastsecond)
            {
                lastsecond=tickdata->time;
                timer-=1;
                std::string message;
                switch(timer)
                {
                case 0:
                    message=std::string("Go!");
                    break;
                case 1:
                    message=std::string("1");
                    break;
                case 2:
                    message=std::string("2");
                    break;
                case 3:
                    message=std::string("3");
                    break;
                case 4:
                    message=std::string("4");
                    break;
                case 5:
                    message=std::string("5");
                    break;
                case 6:
                    message=std::string("6");
                    break;
                case 7:
                    message=std::string("7");
                    break;
                case 8:
                    message=std::string("8");
                    break;
                case 9:
                    message=std::string("9");
                    break;
                case 10:
                    message=std::string("10");
                    break;
                case 11:
                    message=std::string("11");
                    break;
                case 12:
                    message=std::string("12");
                    break;
                case 13:
                    message=std::string("13 seconds untill game start");
                    break;
                case 14:
                    message=std::string("Last chance to join! If you are an observer, type /join to join now!");
                    break;
                }
                bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS,message.c_str());
            }
            if(timer<1)
            {
                bz_debugMessage(1,"ZOMBIEDEBUG: Timer hit zero. Starting team decision making.");
                timer=0;
                bzAPIIntList* playerlist=bz_newIntList();
                bz_getPlayerIndexList(playerlist);
                int num = 0;
                for ( unsigned int i=0; i < playerlist->size(); i++)
                {
                    bz_PlayerRecord *updateData;
                    updateData = bz_getPlayerByIndex(playerlist->get(i));
                    if(updateData->team == eRogueTeam)
                    {
                        if (num%2 == 0)
                        {
                            teamtracker[updateData->playerID] = eGreenTeam;
                            setZombie(updateData->playerID);
                        }
                        else
                        {
                            teamtracker[updateData->playerID] = eRedTeam;
                            setHuman(updateData->playerID);
                        }
                        bz_debugMessagef(1,"ZOMBIEDEBUG: %s confirmed as a player. Decided team: ",updateData->callsign.c_str(), num%2 == 0 ? "green":"red");
                        num++;
                    }
                    bz_killPlayer(updateData->playerID, false, -1);
                    bz_freePlayerRecord(updateData);
                }
                bz_deleteIntList(playerlist);
                gameInProgress = true;
                bz_debugMessage(1,"ZOMBIEDEBUG: Loop done. Game in progress.");
            }
        }
        if((bz_getTeamCount(eRedTeam) == 0 || bz_getTeamCount(eGreenTeam) == 0) && gameInProgress)
        {
            gameInProgress = false;
            bz_debugMessage(1,"ZOMBIEDEBUG: Team won, ending game...");
            bzAPIIntList* playerlist=bz_newIntList();
            bz_getPlayerIndexList(playerlist);
            for ( unsigned int i=0; i < playerlist->size(); i++)
            {
                bz_PlayerRecord *updateData;
                updateData = bz_getPlayerByIndex(playerlist->get(i));

                if(updateData->team != eObservers)
                {
                    dontMove(updateData->playerID);
                    teamtracker[updateData->playerID] = eRogueTeam;
                    bz_killPlayer(updateData->playerID, false, -1);
                    bz_debugMessagef(1,"ZOMBIEDEBUG: %s is in the game. Killed and set as a rogue.",updateData->callsign.c_str());
                }

                bz_freePlayerRecord(updateData);
            }

            bz_deleteIntList(playerlist);

            if(bz_getTeamCount(eRedTeam) == 0)
            {
                bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS,"The zombies have won! Congragulations!");
                bz_debugMessage(1,"ZOMBIEDEBUG: Zombies won the game");
            }
            else if (bz_getTeamCount(eGreenTeam) == 0)
            {
                bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS,"The humans have won! Congragulations!");
                bz_debugMessage(1,"ZOMBIEDEBUG: Humans won the game");
            }
            if(bz_getTeamCount(eRogueTeam) > 2 || bz_getTeamCount(eGreenTeam) > 2 || bz_getTeamCount(eRedTeam) > 2)
            {
                bz_debugMessage(1,"ZOMBIEDEBUG: It seems we have enough players, starting another game.");
                bz_sendTextMessage(BZ_SERVER,BZ_ALLUSERS,"The server has automatically started a new game");
                timer = 15;
            }
        }
        if(tickdata->time-2>lastsec)
        {
            lastsec=tickdata->time;
            bzAPIIntList* playerlist=bz_newIntList();
            bz_getPlayerIndexList(playerlist);
            for ( unsigned int i=0; i < playerlist->size(); i++)
            {
                bz_PlayerRecord *updateData;
                updateData = bz_getPlayerByIndex(playerlist->get(i));

                if(updateData->team != teamtracker[updateData->playerID])
                {
                    bz_debugMessagef(1,"ZOMBIEDEBUG: Team tracker was not in sync for %s. Updating team now...",updateData->callsign.c_str());
                    switch(teamtracker[updateData->playerID])
                    {
                        default:
                            break;
                        case eGreenTeam:
                            setZombie(updateData->playerID);
                            break;
                        case eRedTeam:
                            setHuman(updateData->playerID);
                            break;
                        case eRogueTeam:
                            setRogue(updateData->playerID);
                            break;
                        case eObservers:
                            setObserver(updateData->playerID);
                            break;
                    }
                }
                bz_freePlayerRecord(updateData);
            }
            bz_deleteIntList(playerlist);
        }
    }
    break;
    case bz_ePlayerPartEvent:
    {
        bz_PlayerJoinPartEventData* partdata = (bz_PlayerJoinPartEventData*)eventData;

        teamtracker[partdata->playerID] = eNoTeam;
    }
    break;
    }
}

bool zombieHandler::handle(int playerID, bzApiString command, bzApiString message, bzAPIStringList* /*params*/)
{
    if(command == "start")
    {
        bz_PlayerRecord *player = bz_getPlayerByIndex(playerID);
        if(!gameInProgress)
        {
            if(bz_getTeamCount(eRogueTeam) > 2)
            {
                if(timer == 0)
                {
                    bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "%s has started a new game of zombie", player->callsign.c_str());
                    timer = 15;
                    bz_debugMessagef(1, "%s has started a new game of zombie", player->callsign.c_str());
                }
                else
                {
                    bz_sendTextMessagef(BZ_SERVER, playerID, "%s, you can not start a countdown when a countdown has been started", player->callsign.c_str());
                    bz_debugMessagef(1, "%s has attempted to start a countdown when a countdown was already running", player->callsign.c_str());
                }
            }
            else
            {
                bz_sendTextMessagef(BZ_SERVER, playerID, "%s, you can not start a countdown with less then three players", player->callsign.c_str());
                bz_debugMessagef(1, "%s has attempted to start a countdown with less then three players", player->callsign.c_str());
            }
        }
        else
        {
            bz_sendTextMessagef(BZ_SERVER, playerID, "%s, you can not start a countdown when there is already a game in progress", player->callsign.c_str());
            bz_debugMessagef(1, "%s has attempted to start a countdown when a game is already in progress", player->callsign.c_str());
        }
        bz_freePlayerRecord(player);
    }
    else if(command == "stop")
    {
        bz_PlayerRecord *player = bz_getPlayerByIndex(playerID);

        bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "%s has ended the current game of zombie", player->callsign.c_str());
        bz_debugMessagef(1, "%s ended the current game of zombie", player->callsign.c_str());
        
        bz_freePlayerRecord(player);

        gameInProgress = false;
        timer = 0;

        // Kill all players and make them not move
        bzAPIIntList* playerlist=bz_newIntList();
        bz_getPlayerIndexList(playerlist);

        for ( unsigned int i=0; i < playerlist->size(); i++)
        {
            bz_PlayerRecord *updateData;
            updateData = bz_getPlayerByIndex(playerlist->get(i));

            if(updateData->team != eObservers)
            {
                dontMove(updateData->playerID);
                setRogue(updateData->playerID);
                bz_killPlayer(updateData->playerID, false, -1);
            }

            bz_freePlayerRecord(updateData);
        }
        bz_deleteIntList(playerlist);
    }
    else if(command == "switch")
    {
        bzAPIStringList params;
        params.tokenize(message.c_str(), " ", 0, true);
        if(isAdmin(playerID))
        {
            if(params.size() >= 2)
            {
                int otherPlayerID = getPlayerByCallsign(params.get(0));
                if(otherPlayerID != -1)
                {
                    bz_PlayerRecord *player = bz_getPlayerByIndex(playerID);
                    if(params.get(1) == "red" || params.get(1) == "green" || params.get(1) == "rogue" || params.get(1) == "observer")
                    {
                        if(params.get(1) == "red")
                        {
                            teamtracker[playerID] = eRedTeam;
                            setHuman(playerID);
                        }
                        if(params.get(1) == "green")
                        {
                            teamtracker[playerID] = eGreenTeam;
                            setZombie(playerID);
                        }
                        if(params.get(1) == "rogue")
                        {
                            teamtracker[playerID] = eRogueTeam;
                            setRogue(playerID);
                        }
                        if(params.get(1) == "observer")
                        {
                            teamtracker[playerID] = eObservers;
                            setObserver(playerID);
                        }

                        bz_sendTextMessagef(BZ_SERVER, eAdministrators, "%s switched %s to the %s team", player->callsign.c_str(), params.get(0).c_str(), params.get(1).c_str());

                        bz_freePlayerRecord(player);
                    }
                    else
                    {
                        bz_sendTextMessagef(BZ_SERVER, playerID, "Not a valid team: %s", params.get(1).c_str());
                    }
                }
                else
                {
                    bz_sendTextMessagef(BZ_SERVER, playerID, "No Such Player: %s", params.get(0).c_str());
                }
            }
            else
            {
                bz_sendTextMessage(BZ_SERVER, playerID, "Usage: /switch <player> <team>");
            }
        }
        else
        {
            bz_sendTextMessage(BZ_SERVER, playerID, "You do not have permission to switch teams");
        }

        return true;
    }
    else if(command == "restart")
    {
    }
    else if(command == "join")
    {
        bz_PlayerRecord *player = bz_getPlayerByIndex(playerID);
        if(player->team == eObservers)
        {
            if(!gameInProgress)
            {
                setRogue(playerID);
                teamtracker[playerID] = eRogueTeam;
                bz_sendTextMessage(BZ_SERVER,playerID,"The game will start shortly. Good luck!");
            }
            else
            {
                bz_sendTextMessage(BZ_SERVER,playerID,"A game is in progress, please wait until it finishes before joining.");
            }
        }
        else
        {
            bz_sendTextMessage(BZ_SERVER,playerID,"You must be an observer to use this command");
        }
    }
    else if(command == "debug")
    {
        bz_sendTextMessage(BZ_SERVER, playerID, "--------( Server Stats )--------");
        bz_sendTextMessagef(BZ_SERVER, playerID, "Game running: %s",gameInProgress ? "Yes":"No");
        bz_sendTextMessagef(BZ_SERVER, playerID, "Countdown in progress: %s",timer>0 ? "Yes":"No");
        bz_sendTextMessagef(BZ_SERVER, playerID, "Countdown time: %d",timer);
        bz_sendTextMessagef(BZ_SERVER, playerID, "Countdown last second: %d",lastsecond);
        bz_sendTextMessage(BZ_SERVER, playerID, "--------( Player Stats )--------");
        bz_sendTextMessagef(BZ_SERVER, playerID, "Rogue players: %d",bz_getTeamCount(eRogueTeam));
        bz_sendTextMessagef(BZ_SERVER, playerID, "Green players: %d",bz_getTeamCount(eGreenTeam));
        bz_sendTextMessagef(BZ_SERVER, playerID, "Red players: %d",bz_getTeamCount(eRedTeam));
        bz_sendTextMessagef(BZ_SERVER, playerID, "Observers: %d",bz_getTeamCount(eObservers));
        bz_sendTextMessage(BZ_SERVER, playerID, "--------( TeamTracker Stats )--------");
        bzAPIIntList* playerlist=bz_newIntList();
        bz_getPlayerIndexList(playerlist);
        for ( unsigned int i=0; i < playerlist->size(); i++)
        {
            bz_PlayerRecord *updateData;
            updateData = bz_getPlayerByIndex(playerlist->get(i));
            
            switch(teamtracker[updateData->playerID])
            {
                default:
                    break;
                case eGreenTeam:
                    bz_sendTextMessagef(BZ_SERVER, playerID, "%d - %s - green",updateData->playerID,updateData->callsign.c_str());
                    break;
                case eRedTeam:
                    bz_sendTextMessagef(BZ_SERVER, playerID, "%d - %s - red",updateData->playerID,updateData->callsign.c_str());
                    break;
                case eRogueTeam:
                    bz_sendTextMessagef(BZ_SERVER, playerID, "%d - %s - rogue",updateData->playerID,updateData->callsign.c_str());
                    break;
                case eObservers:
                    bz_sendTextMessagef(BZ_SERVER, playerID, "%d - %s - observer",updateData->playerID,updateData->callsign.c_str());
                    break;
            }
            bz_freePlayerRecord(updateData);
        }
        bz_deleteIntList(playerlist);        
    }
    return true;
}

BZF_PLUGIN_CALL int bz_Unload()
{
    bz_removeEvent(bz_ePlayerDieEvent, &zombie);
    bz_removeEvent(bz_ePlayerSpawnEvent, &zombie);
    bz_removeEvent(bz_ePlayerJoinEvent, &zombie);
    bz_removeEvent(bz_eGetPlayerSpawnPosEvent, &zombie);
    bz_removeEvent(bz_eFlagDroppedEvent, &zombie);
    bz_removeEvent(bz_eTickEvent, &zombie);
    bz_removeEvent(bz_eGetAutoTeamEvent, &zombie);
    bz_removeEvent(bz_ePlayerPartEvent, &zombie);

    bz_removeCustomSlashCommand ("start");
    bz_removeCustomSlashCommand ("stop");
    bz_removeCustomSlashCommand ("switch");
    bz_removeCustomSlashCommand ("restart");
    bz_removeCustomSlashCommand ("join");
    bz_removeCustomSlashCommand ("debug");

    bz_debugMessage(4,"zombie plugin unloaded");

    return 0;
}

// Local Variables: ***
// mode:C++ ***
// tab-width: 8 ***
// c-basic-offset: 2 ***
// indent-tabs-mode: t ***
// End: ***
// ex: shiftwidth=2 tabstop=8